const colorNames = require("./colorNames")
const themeDefaults = require("./themeDefaults")

const { toGamut, interpolate, wcagContrast } = require("culori")

module.exports = {
  changeColorValuesToObject: function (input) {
    const [h, s, l] = input.match(/\d+(\.\d+)?%|\d+(\.\d+)?/g).map(parseFloat)
    return { h, s, l, a: 1 }
  },

  isDark: (color) => {
    if (wcagContrast(color, "black") < wcagContrast(color, "white")) {
      return true
    }
    return false
  },

  colorObjToString: function (input, colorSpace) {
    const cut = (number) => {
      if (!number) {
        return 0
      }
      return +number.toFixed(4)
    }
    if (colorSpace === "oklch") {
      const { l, c, h } = input
      return `${cut(l)} ${cut(c)} ${cut(h)}`
    }
    if (colorSpace === "hsl") {
      const { h, s, l } = input
      return `${cut(h)} ${cut(s)}% ${cut(l)}%`
    }
  },

  generateForegroundColorFrom: function (input, percentage = 0.8, colorSpace) {
    const result = interpolate(
      [input, this.isDark(input) ? "white" : "black"],
      colorSpace
    )(percentage)
    return this.colorObjToString(result, colorSpace)
  },

  generateDarkenColorFrom: function (input, percentage = 0.07, colorSpace) {
    const result = interpolate([input, "black"], colorSpace)(percentage)
    return this.colorObjToString(result, colorSpace)
  },

  convertColorFormat: function (input, colorSpace) {
    if (typeof input !== "object" || input === null) {
      return input
    }

    const resultObj = {}

    Object.entries(input).forEach(([rule, value]) => {
      if (colorNames.hasOwnProperty(rule)) {
        const colorObj = toGamut(colorSpace)(value)
        resultObj[colorNames[rule]] = this.colorObjToString(colorObj, colorSpace)
      } else {
        resultObj[rule] = value
      }

      // auto generate focus colors
      if (!Object.hasOwn(input, "primary-focus")) {
        resultObj["--pf"] = this.generateDarkenColorFrom(input["primary"], 0.07, colorSpace)
      }
      if (!Object.hasOwn(input, "secondary-focus")) {
        resultObj["--sf"] = this.generateDarkenColorFrom(input["secondary"], 0.07, colorSpace)
      }
      if (!Object.hasOwn(input, "accent-focus")) {
        resultObj["--af"] = this.generateDarkenColorFrom(input["accent"], 0.07, colorSpace)
      }
      if (!Object.hasOwn(input, "neutral-focus")) {
        resultObj["--nf"] = this.generateDarkenColorFrom(input["neutral"], 0.07, colorSpace)
      }

      // auto generate base colors
      if (!Object.hasOwn(input, "base-100")) {
        resultObj["--b1"] = "100 0 0"
      }
      if (!Object.hasOwn(input, "base-200")) {
        resultObj["--b2"] = this.generateDarkenColorFrom(input["base-100"], 0.07, colorSpace)
      }
      if (!Object.hasOwn(input, "base-300")) {
        if (Object.hasOwn(input, "base-200")) {
          resultObj["--b3"] = this.generateDarkenColorFrom(input["base-200"], 0.07, colorSpace)
        } else {
          resultObj["--b3"] = this.generateDarkenColorFrom(input["base-100"], 0.14, colorSpace)
        }
      }

      // auto generate state colors

      if (!Object.hasOwn(input, "info")) {
        resultObj["--in"] = "0.7206 0.191 231.6"
      }
      if (!Object.hasOwn(input, "success")) {
        resultObj["--su"] = "0.7441 0.213 164.75"
      }
      if (!Object.hasOwn(input, "warning")) {
        resultObj["--wa"] = "0.8471 0.199 83.87"
      }
      if (!Object.hasOwn(input, "error")) {
        resultObj["--er"] = "0.7176 0.221 22.18"
      }

      // auto generate content colors
      if (!Object.hasOwn(input, "base-content")) {
        resultObj["--bc"] = this.generateForegroundColorFrom(input["base-100"], 0.8, colorSpace)
      }
      if (!Object.hasOwn(input, "primary-content")) {
        resultObj["--pc"] = this.generateForegroundColorFrom(input["primary"], 0.8, colorSpace)
      }
      if (!Object.hasOwn(input, "secondary-content")) {
        resultObj["--sc"] = this.generateForegroundColorFrom(input["secondary"], 0.8, colorSpace)
      }
      if (!Object.hasOwn(input, "accent-content")) {
        resultObj["--ac"] = this.generateForegroundColorFrom(input["accent"], 0.8, colorSpace)
      }
      if (!Object.hasOwn(input, "neutral-content")) {
        resultObj["--nc"] = this.generateForegroundColorFrom(input["neutral"], 0.8, colorSpace)
      }
      if (!Object.hasOwn(input, "info-content")) {
        if (Object.hasOwn(input, "info")) {
          resultObj["--inc"] = this.generateForegroundColorFrom(input["info"], 0.8, colorSpace)
        } else {
          resultObj["--inc"] = "0 0 0"
        }
      }
      if (!Object.hasOwn(input, "success-content")) {
        if (Object.hasOwn(input, "success")) {
          resultObj["--suc"] = this.generateForegroundColorFrom(input["success"], 0.8, colorSpace)
        } else {
          resultObj["--suc"] = "0 0 0"
        }
      }
      if (!Object.hasOwn(input, "warning-content")) {
        if (Object.hasOwn(input, "warning")) {
          resultObj["--wac"] = this.generateForegroundColorFrom(input["warning"], 0.8, colorSpace)
        } else {
          resultObj["--wac"] = "0 0 0"
        }
      }
      if (!Object.hasOwn(input, "error-content")) {
        if (Object.hasOwn(input, "error")) {
          resultObj["--erc"] = this.generateForegroundColorFrom(input["error"], 0.8, colorSpace)
        } else {
          resultObj["--erc"] = "0 0 0"
        }
      }

      // add css variables if not exist
      Object.entries(themeDefaults.variables).forEach((item) => {
        const [variable, value] = item
        if (!Object.hasOwn(input, variable)) {
          resultObj[variable] = value
        }
      })

      // add other custom styles
      if (!colorNames.hasOwnProperty(rule)) {
        resultObj[rule] = value
      }
    })

    return resultObj
  },

  injectThemes: function (addBase, config, themes) {
    let colorSpace = "oklch"
    // if (["oklch", "hsl"].includes(config("daisyui.colorSpace"))) {
    //   colorSpace = config("daisyui.colorSpace")
    // }
    const includedThemesObj = {}
    // add default themes
    Object.entries(themes).forEach(([theme, value]) => {
      includedThemesObj[theme] = this.convertColorFormat(value, colorSpace)
    })

    // add custom themes
    if (Array.isArray(config("daisyui.themes"))) {
      config("daisyui.themes").forEach((item) => {
        if (typeof item === "object" && item !== null) {
          Object.entries(item).forEach(([customThemeName, customThemevalue]) => {
            includedThemesObj["[data-theme=" + customThemeName + "]"] = this.convertColorFormat(
              customThemevalue,
              colorSpace
            )
          })
        }
      })
    }

    let themeOrder = []
    if (Array.isArray(config("daisyui.themes"))) {
      config("daisyui.themes").forEach((theme) => {
        if (typeof theme === "object" && theme !== null) {
          Object.keys(theme).forEach((customThemeName) => {
            themeOrder.push(customThemeName)
          })
        } else if (Object.hasOwn(includedThemesObj, "[data-theme=" + theme + "]")) {
          themeOrder.push(theme)
        }
      })
    } else if (config("daisyui.themes") === true) {
      themeOrder = themeDefaults.themeOrder
    } else {
      themeOrder = ["light", "dark"]
    }

    // inject themes in order
    const themesToInject = {}
    themeOrder.forEach((themeName, index) => {
      if (index === 0) {
        // first theme as root
        themesToInject[":root"] = includedThemesObj["[data-theme=" + themeName + "]"]
      } else if (index === 1) {
        // auto dark
        if (config("daisyui.darkTheme")) {
          if (
            themeOrder[0] !== config("daisyui.darkTheme") &&
            themeOrder.includes(config("daisyui.darkTheme"))
          ) {
            themesToInject["@media (prefers-color-scheme: dark)"] = {
              [":root"]: includedThemesObj[`[data-theme=${config("daisyui.darkTheme")}]`],
            }
          }
        } else if (config("daisyui.darkTheme") === false) {
          // disables prefers-color-scheme: dark
        } else {
          if (themeOrder[0] !== "dark" && themeOrder.includes("dark")) {
            themesToInject["@media (prefers-color-scheme: dark)"] = {
              [":root"]: includedThemesObj["[data-theme=dark]"],
            }
          }
        }
        // theme 0 with name
        themesToInject["[data-theme=" + themeOrder[0] + "]"] =
          includedThemesObj["[data-theme=" + themeOrder[0] + "]"]
        // theme 1 with name
        themesToInject["[data-theme=" + themeOrder[1] + "]"] =
          includedThemesObj["[data-theme=" + themeOrder[1] + "]"]
      } else {
        themesToInject["[data-theme=" + themeName + "]"] =
          includedThemesObj["[data-theme=" + themeName + "]"]
      }
    })

    addBase(themesToInject)

    return {
      includedThemesObj,
      themeOrder,
    }
  },
}
